@@ -24,9 +24,9 @@ from typing import (
     overload,
 )
 
-from aiohttp.web import Application, AppRunner
-from aiohttp_json_rpc import JsonRpc, RpcError, RpcGenericServerDefinedError, RpcInvalidParamsError
-from aiohttp_json_rpc.rpc import JsonRpcMethod
+from aiohttp import web
+from aiohttp_rpc import JsonRpcMethod, WsJsonRpcServer, middlewares as rpc_middlewares
+from aiohttp_rpc.errors import InvalidParams as InvalidParamsError, ServerError
 from pydantic import BaseModel, Field, ValidationError, validator
 
 from . import exceptions as E
@@ -45,34 +45,19 @@ if TYPE_CHECKING:
 API_VERSION = 0
 
 
-class _InstawowError(RpcGenericServerDefinedError):
-    ERROR_CODE = -32000
-    MESSAGE = 'instawow encountered an error'
-
-
-class _ConfigError(_InstawowError):
-    ERROR_CODE = -32001
-    MESSAGE = 'invalid configuration parameters'
+class _ConfigError(ServerError):
+    code = -32001
+    message = 'invalid configuration parameters'
 
 
 @contextmanager
-def _reraise_validation_error(error_class: Type[RpcError] = _InstawowError) -> Iterator[None]:
+def _reraise_validation_error(error_class: Type[ServerError] = ServerError) -> Iterator[None]:
     try:
         yield
     except ValidationError as error:
         raise error_class(data=error.errors()) from error
 
 
-JSONRPC = '2.0'
-
-
-class Request(BaseModel):
-    jsonrpc = JSONRPC
-    method: str
-    params: Union[List[Any], Dict[str, Any]]
-    id: Union[None, int, str]
-
-
 class BaseParams(BaseModel):
     _method: ClassVar[str]
 
@@ -90,7 +75,7 @@ class _DefnParamMixin(BaseModel):
 
 class WriteConfigParams(BaseParams):
     values: Dict[str, Any]
-    _method = 'config.write'
+    _method = 'config/write'
     _result_type = Config
 
     @t
@@ -105,7 +90,7 @@ class WriteConfigParams(BaseParams):
 
 
 class ReadConfigParams(_ProfileParamMixin, BaseParams):
-    _method = 'config.read'
+    _method = 'config/read'
     _result_type = Config
 
     @t
@@ -115,7 +100,7 @@ class ReadConfigParams(_ProfileParamMixin, BaseParams):
 
 
 class DeleteConfigParams(_ProfileParamMixin, BaseParams):
-    _method = 'config.delete'
+    _method = 'config/delete'
     _result_type = type(None)
 
     async def respond(self, managers: ManagerWorkQueue) -> _result_type:
@@ -127,7 +112,7 @@ class DeleteConfigParams(_ProfileParamMixin, BaseParams):
 
 
 class EnumerateProfilesParams(BaseParams):
-    _method = 'config.enumerate'
+    _method = 'config/enumerate'
     _result_type = List[str]
 
     @t
@@ -150,7 +135,7 @@ class _Source(BaseModel):
 
 
 class ListSourcesParams(_ProfileParamMixin, BaseParams):
-    _method = 'sources.list'
+    _method = 'sources/list'
     _result_type = List[_Source]
 
     async def respond(self, managers: ManagerWorkQueue) -> _result_type:
@@ -158,52 +143,19 @@ class ListSourcesParams(_ProfileParamMixin, BaseParams):
         return list(map(_Source.from_orm, manager.resolvers.values()))
 
 
-class _PkgMeta(BaseModel):
-    installed: bool = False
-    damaged: bool = False
-    new_version: O[str] = None
-
-
 class _ListResult(BaseModel):
-    __root__: List[Tuple[PkgModel, _PkgMeta]]
+    __root__: List[O[PkgModel]]
 
 
 class ListInstalledParams(_ProfileParamMixin, BaseParams):
-    check_for_updates: bool
     _method = 'list'
     _result_type = _ListResult
 
     async def respond(self, managers: ManagerWorkQueue) -> _result_type:
-        manager = await managers.run(self.profile)
-        installed_pkgs = manager.database.query(Pkg).all()
-        damaged_pkgs = await managers.run(self.profile, t(Manager.find_damaged_pkgs))
-        if self.check_for_updates:
-            resolve_results = await managers.run(
-                self.profile,
-                partial(Manager.resolve, defns=list(map(Defn.from_pkg, installed_pkgs))),
-            )
-            outdated_pkgs = {
-                p: r.version
-                for p, r in zip(installed_pkgs, resolve_results.values())
-                if is_pkg(r) and p.version != r.version
-            }
-        else:
-            outdated_pkgs = {}
-        pkg_objs = [
-            (
-                p,
-                _PkgMeta(
-                    installed=True, damaged=p in damaged_pkgs, new_version=outdated_pkgs.get(p)
-                ),
-            )
-            for p in installed_pkgs
-        ]
-        pkg_objs = sorted(
-            pkg_objs,
-            key=lambda i: (not i[1].damaged, not i[1].new_version, i[0].name),
+        installed_pkgs = await managers.run(
+            self.profile, t(lambda m: m.database.query(Pkg).order_by(Pkg.name).all())
         )
-        result = _ListResult.parse_obj(pkg_objs)
-        return result
+        return _ListResult.parse_obj(installed_pkgs)
 
 
 class SearchParams(_ProfileParamMixin, BaseParams):
@@ -223,7 +175,7 @@ class SearchParams(_ProfileParamMixin, BaseParams):
                 strategy=self.strategy,
             ),
         )
-        return _ListResult.parse_obj([(r, _PkgMeta()) for r in results.values()])
+        return _ListResult.parse_obj(list(results.values()))
 
 
 class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
@@ -231,28 +183,29 @@ class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
     _result_type = _ListResult
 
     async def respond(self, managers: ManagerWorkQueue) -> _result_type:
-        results = await managers.run(self.profile, partial(Manager.resolve, defns=self.defns))
-        return _ListResult.parse_obj([(r, _PkgMeta()) for r in results.values() if is_pkg(r)])
+        def extract_source(manager: Manager, defns: List[Defn]):
+            for defn in defns:
+                if defn.source == '*':
+                    pair = manager.pair_uri(defn.alias)
+                    if pair:
+                        source, alias = pair
+                        defn = defn.with_(source=source, alias=alias)
+                yield defn
 
-
-class ResolveUrisParams(_ProfileParamMixin, BaseParams):
-    prospective_defns: List[str]
-    strategy: Strategies = Strategies.default
-    _method = 'resolve_uris'
-    _result_type = _ListResult
-
-    async def respond(self, managers: ManagerWorkQueue) -> _result_type:
-        manager = await managers.run(self.profile)
-        source_name_pairs = filter(None, map(manager.pair_uri, self.prospective_defns))
-        defns = [Defn.get(a, b).with_(strategy=self.strategy) for a, b in source_name_pairs]
-        results = await managers.run(self.profile, partial(Manager.resolve, defns=defns))
-        return _ListResult.parse_obj([(r, _PkgMeta()) for r in results.values() if is_pkg(r)])
+        results = await managers.run(
+            self.profile,
+            partial(
+                Manager.resolve,
+                defns=list(extract_source(await managers.run(self.profile), self.defns)),
+            ),
+        )
+        return _ListResult.parse_obj([r if is_pkg(r) else None for r in results.values()])
 
 
 class _ModifyResult(BaseModel):
     __root__: List[
         Union[
-            Tuple[Literal['success'], Tuple[PkgModel, _PkgMeta]],
+            Tuple[Literal['success'], PkgModel],
             Tuple[Literal['failure', 'error'], str],
         ]
     ]
@@ -269,12 +222,7 @@ class InstallParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
         )
         return _ModifyResult.parse_obj(
             [
-                (
-                    r.kind,
-                    (r.pkg, _PkgMeta(installed=True))
-                    if isinstance(r, E.PkgInstalled)
-                    else r.message,
-                )
+                (r.kind, r.pkg if isinstance(r, E.PkgInstalled) else r.message)
                 for r in results.values()
             ]
         )
@@ -285,15 +233,12 @@ class UpdateParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
     _result_type = _ModifyResult
 
     async def respond(self, managers: ManagerWorkQueue) -> _result_type:
-        results = await managers.run(self.profile, partial(Manager.update, defns=self.defns))
+        results = await managers.run(
+            self.profile, partial(Manager.update, defns=self.defns, retain_strategy=True)
+        )
         return _ModifyResult.parse_obj(
             [
-                (
-                    r.kind,
-                    (r.new_pkg, _PkgMeta(installed=True))
-                    if isinstance(r, E.PkgUpdated)
-                    else r.message,
-                )
+                (r.kind, r.new_pkg if isinstance(r, E.PkgUpdated) else r.message)
                 for r in results.values()
             ]
         )
@@ -307,7 +252,21 @@ class RemoveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
         results = await managers.run(self.profile, partial(Manager.remove, defns=self.defns))
         return _ModifyResult.parse_obj(
             [
-                (r.kind, (r.old_pkg, _PkgMeta()) if isinstance(r, E.PkgRemoved) else r.message)
+                (r.kind, r.old_pkg if isinstance(r, E.PkgRemoved) else r.message)
+                for r in results.values()
+            ]
+        )
+
+
+class PinParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
+    _method = 'pin'
+    _result_type = _ModifyResult
+
+    async def respond(self, managers: ManagerWorkQueue) -> _result_type:
+        results = await managers.run(self.profile, partial(Manager.pin, defns=self.defns))
+        return _ModifyResult.parse_obj(
+            [
+                (r.kind, r.pkg if isinstance(r, E.PkgInstalled) else r.message)
                 for r in results.values()
             ]
         )
@@ -368,35 +327,22 @@ class ReconcileParams(_ProfileParamMixin, BaseParams):
         return _ReconcileResult(reconciled=reconciled, unreconciled=unreconciled)
 
 
-class _GetVersionResponse(BaseModel):
+class _GetVersionResult(BaseModel):
     installed_version: str
     new_version: O[str]
 
 
 class GetVersionParams(BaseParams):
-    _method = 'meta.get_version'
-    _result_type = _GetVersionResponse
+    _method = 'meta/get_version'
+    _result_type = _GetVersionResult
 
     async def respond(self, managers: ManagerWorkQueue) -> _result_type:
         outdated, new_version = await t(is_outdated)()
-        return _GetVersionResponse(
+        return _GetVersionResult(
             installed_version=get_version(), new_version=new_version if outdated else None
         )
 
 
-class _Response(BaseModel):
-    jsonrpc = JSONRPC
-    id: Union[None, int, str]
-
-
-class SuccessResponse(_Response):
-    result: Any
-
-
-class ErrorResponse(_Response):
-    error: Any
-
-
 class ManagerWorkQueue:
     def __init__(self) -> None:
         asyncio.get_running_loop()  # Sanity check
@@ -464,27 +410,26 @@ class ManagerWorkQueue:
         self._managers.pop(profile, None)
 
 
-def _prepare_response(
-    param_class: Type[BaseParams], managers: ManagerWorkQueue
-) -> Tuple[str, JsonRpcMethod]:
-    async def respond(request: Any) -> Any:
-        with _reraise_validation_error(RpcInvalidParamsError):
-            params = param_class.parse_obj(request.params)
+def _prepare_response(param_class: Type[BaseParams], managers: ManagerWorkQueue) -> JsonRpcMethod:
+    async def respond(**kwargs: Any) -> BaseModel:
+        with _reraise_validation_error(InvalidParamsError):
+            params = param_class.parse_obj(kwargs)
+        return await params.respond(managers)
 
-        response = SuccessResponse(
-            result=await params.respond(managers), id=request.msg.data['id']
-        )
-        return response.json()
-
-    # Signal to ``JsonRpc`` that it should not attempt to encode the return value
-    respond.raw_response = True
-    return (param_class._method, JsonRpcMethod(respond))
+    return JsonRpcMethod('', respond, custom_name=param_class._method)
 
 
-async def create_app() -> Application:
+def serialise_response(value: Dict[str, Any]) -> str:
+    return BaseModel.construct(**value).json()
+
+
+async def create_app() -> web.Application:
     managers = ManagerWorkQueue()
-    json_rpc = JsonRpc()
-    json_rpc.methods = dict(
+    rpc_server = WsJsonRpcServer(
+        json_serialize=serialise_response,
+        middlewares=rpc_middlewares.DEFAULT_MIDDLEWARES,
+    )
+    rpc_server.add_methods(
         map(
             _prepare_response,
             (
@@ -496,24 +441,20 @@ async def create_app() -> Application:
                 ListInstalledParams,
                 SearchParams,
                 ResolveParams,
-                ResolveUrisParams,
                 InstallParams,
                 UpdateParams,
                 RemoveParams,
+                PinParams,
                 ReconcileParams,
                 GetVersionParams,
             ),
             repeat(managers),
         )
     )
-    app = Application()
-    app.router.add_route(
-        '*',
-        f'/v{API_VERSION}',
-        json_rpc.handle_request,  # type: ignore  # aiohttp_json_rpc is untyped
-    )
+    app = web.Application()
+    app.router.add_routes([web.get(f'/v{API_VERSION}', rpc_server.handle_http_request)])
 
-    async def on_shutdown(app: Application):
+    async def on_shutdown(app: web.Application):
         listen.cancel()
         await managers.cleanup()
 
@@ -525,7 +466,7 @@ async def create_app() -> Application:
 async def listen() -> None:
     "Fire up the server."
     loop = asyncio.get_running_loop()
-    app_runner = AppRunner(await create_app())
+    app_runner = web.AppRunner(await create_app())
     await app_runner.setup()
     # Placate the type checker - server is created in ``app_runner.setup``
     assert app_runner.server
