@@ -14,20 +14,21 @@ from typing import (
     Optional as O,
     Sequence,
     Set,
-    Tuple,
+    Union,
     cast,
 )
 
-from pydantic import BaseModel
+from pydantic import BaseModel, WrongConstantError, validator
+from typing_extensions import Literal, TypedDict, get_args
 from yarl import URL
 
 from . import exceptions as E, models as m
-from .utils import Literal, cached_property, gather, run_in_thread as t, slugify, uniq
+from .utils import cached_property, gather, run_in_thread as t, slugify, uniq
 
 if TYPE_CHECKING:
-    from .manager import Manager
+    from pydantic.fields import ModelField
 
-    JsonDict = Dict[str, Any]
+    from .manager import Manager
 
 
 class Strategies(enum.Enum):
@@ -39,42 +40,74 @@ class Strategies(enum.Enum):
     version = 'version'
 
 
-class Defn(BaseModel):
+class HashableModel(BaseModel):
+    def __eq__(self, other: Any) -> bool:
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+        return self.__dict__ == other.__dict__
+
+    def __hash__(self) -> int:
+        return hash(tuple(self.__dict__.items()))
+
+
+class BaseStrategy(HashableModel):
+    @validator('type_', check_fields=False, pre=True)
+    def _parse_strategy(cls, value: Any, field: ModelField) -> Strategies:
+        accepted_strategies = get_args(field.type_)
+        strategy = Strategies.__members__.get(value) or value
+        if strategy in accepted_strategies:
+            return strategy
+        raise WrongConstantError(given=value, permitted=accepted_strategies)
+
+
+class SimpleStrategy(BaseStrategy):
+    type_: Literal[
+        Strategies.default,
+        Strategies.latest,
+        Strategies.curse_latest_beta,
+        Strategies.curse_latest_alpha,
+        Strategies.any_flavour,
+    ]
+
+
+class VersionStrategy(BaseStrategy):
+    type_: Literal[Strategies.version]
+    version: str
+
+
+class Defn(HashableModel):
     source: str
     source_id: O[str] = None
-    name: str
-    strategy: Strategies = Strategies.default
-    strategy_vals: Tuple[str, ...] = ()
+    alias: str
+    strategy: Union[SimpleStrategy, VersionStrategy] = SimpleStrategy(type_=Strategies.default)
 
     @classmethod
     def from_pkg(cls, pkg: m.Pkg) -> Defn:
-        defn = cls(
-            source=pkg.source, source_id=pkg.id, name=pkg.slug, strategy=pkg.options.strategy
+        strategy_kwargs: Dict[str, str] = (
+            {'version': pkg.version} if pkg.options.strategy == 'version' else {}
+        )
+        return cls(
+            source=pkg.source,
+            source_id=pkg.id,
+            alias=pkg.slug,
+            strategy={'type_': pkg.options.strategy, **strategy_kwargs},
         )
-        if defn.strategy is Strategies.version:
-            defn.strategy_vals = (pkg.version,)
-        return defn
 
     @classmethod
-    def get(cls, source: str, name: str) -> Defn:
-        return cls(source=source, name=name)
+    def get(cls, source: str, alias: str) -> Defn:
+        return cls(source=source, alias=alias)
 
     def with_(self, **kwargs: Any) -> Defn:
         return self.__class__(**{**self.__dict__, **kwargs})
 
+    def with_strategy(self, strategy: Strategies) -> Defn:
+        return self.with_(strategy={'type_': strategy})
+
     def with_version(self, version: str) -> Defn:
-        return self.with_(strategy='version', strategy_vals=(version,))
-
-    def __eq__(self, other: Any) -> bool:
-        if isinstance(other, self.__class__):
-            other = tuple(other.__dict__.values())
-        return tuple(self.__dict__.values()) == other
-
-    def __hash__(self) -> int:
-        return hash(tuple(self.__dict__.values()))
+        return self.with_(strategy={'type_': Strategies.version, 'version': version})
 
     def __str__(self) -> str:
-        return f'{self.source}:{self.name}'
+        return f'{self.source}:{self.alias}'
 
 
 class _CatalogueEntry(BaseModel):
@@ -120,10 +153,10 @@ class Resolver:
         self.manager = manager
 
     def __init_subclass__(cls) -> None:
-        async def resolve_wrapper(self: Resolver, defn: Defn, metadata: O[JsonDict]) -> m.Pkg:
-            if defn.strategy in self.strategies:
+        async def resolve_wrapper(self: Resolver, defn: Defn, metadata: O[Any]) -> m.Pkg:
+            if defn.strategy.type_ in self.strategies:
                 return await resolve_one(self, defn, metadata)  # type: ignore
-            raise E.PkgStrategyUnsupported(defn.strategy)
+            raise E.PkgStrategyUnsupported(defn.strategy.type_)
 
         resolve_one = cls.resolve_one
         cls.resolve_one = resolve_wrapper  # type: ignore
@@ -134,7 +167,7 @@ class Resolver:
         return Strategies.version in self.strategies
 
     @staticmethod
-    def get_name_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> O[str]:
         "Attempt to extract a definition name from a given URL."
 
     async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
@@ -142,7 +175,7 @@ class Resolver:
         results = await gather(self.resolve_one(d, None) for d in defns)
         return dict(zip(defns, results))
 
-    async def resolve_one(self, defn: Defn, metadata: O[JsonDict]) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: O[Any]) -> m.Pkg:
         "Resolve an individual definition into a package."
         raise NotImplementedError
 
@@ -152,6 +185,50 @@ class Resolver:
         yield
 
 
+if TYPE_CHECKING:
+
+    # Only documenting the fields we're actually using -
+    # the API responses are absolutely massive
+
+    class CurseAddon_FileDependency(TypedDict):
+        id: int  # Unique dependency ID
+        addonId: int  # The ID of the add-on we're depending on
+        # The type of dependency.  One of:
+        #   1 = embedded library
+        #   2 = optional dependency
+        #   3 = required dependency (this is the one we're after)
+        #   4 = tool
+        #   5 = incompatible
+        #   6 = include (wat)
+        type: int
+        fileId: int  # The ID of the parent file which has this as a dependency
+
+    class CurseAddon_FileModules(TypedDict):
+        foldername: str
+        fingerprint: int
+        type: int
+
+    class CurseAddon_File(TypedDict):
+        id: int  # Unique file ID
+        displayName: str  # Tends to be the version
+        downloadUrl: str
+        fileDate: str  # Upload datetime in ISO, e.g. '2020-02-02T12:12:12Z'
+        releaseType: int  # 1 = stable; 2 = beta; 3 = alpha
+        dependencies: List[CurseAddon_FileDependency]
+        modules: List[CurseAddon_FileModules]
+        exposeAsAlternative: O[bool]
+        gameVersion: List[str]  # e.g. '8.3.0'
+        gameVersionFlavor: Literal['wow_classic', 'wow_retail']
+
+    class CurseAddon(TypedDict):
+        id: int
+        name: str  # User-facing add-on name
+        websiteUrl: str  # e.g. 'https://www.curseforge.com/wow/addons/molinari'
+        summary: str  # One-line description of the add-on
+        latestFiles: List[CurseAddon_File]
+        slug: str  # URL slug; 'molinari' in 'https://www.curseforge.com/wow/addons/molinari'
+
+
 class CurseResolver(Resolver):
     source = 'curse'
     name = 'CurseForge'
@@ -168,7 +245,7 @@ class CurseResolver(Resolver):
     addon_api_url = URL('https://addons-ecs.forgesvc.net/api/v2/addon')
 
     @staticmethod
-    def get_name_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> O[str]:
         url = URL(value)
         if url.host == 'www.wowace.com' and len(url.parts) > 2 and url.parts[1] == 'projects':
             return url.parts[2].lower()
@@ -184,11 +261,11 @@ class CurseResolver(Resolver):
 
         from .manager import cache_json_response
 
-        ids_for_defns = {
-            d: (d.source_id or self.manager.catalogue.curse_slugs.get(d.name) or d.name)
+        defns_to_ids = {
+            d: (d.source_id or self.manager.catalogue.curse_slugs.get(d.alias) or d.alias)
             for d in defns
         }
-        numeric_ids = {i for i in ids_for_defns.values() if i.isdigit()}
+        numeric_ids = {i for i in defns_to_ids.values() if i.isdigit()}
         try:
             json_response = await cache_json_response(
                 self.manager,
@@ -203,15 +280,15 @@ class CurseResolver(Resolver):
 
         api_results = {str(r['id']): r for r in json_response}
         results = await gather(
-            self.resolve_one(d, api_results.get(i)) for d, i in ids_for_defns.items()
+            self.resolve_one(d, api_results.get(i)) for d, i in defns_to_ids.items()
         )
         return dict(zip(defns, results))
 
-    async def resolve_one(self, defn: Defn, metadata: O[JsonDict]) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: O[CurseAddon]) -> m.Pkg:
         if not metadata:
             raise E.PkgNonexistent
 
-        if defn.strategy is Strategies.version:
+        if defn.strategy.type_ is Strategies.version:
 
             from .manager import cache_json_response
 
@@ -222,29 +299,29 @@ class CurseResolver(Resolver):
                 label=f'Fetching metadata from {self.name}',
             )
 
-            def is_compatible(f: JsonDict):
-                return defn.strategy_vals[0] == f['displayName']
+            def is_compatible(f: CurseAddon_File):
+                return cast(VersionStrategy, defn.strategy).version == f['displayName']
 
         else:
 
             files = metadata['latestFiles']
 
-            def is_not_libless(f: JsonDict):
+            def is_not_libless(f: CurseAddon_File):
                 # There's also an 'isAlternate' field that's missing from some
                 # 50 lib-less files from c. 2008.  'exposeAsAlternative' is
                 # absent from the /file endpoint
                 return not f['exposeAsAlternative']
 
-            if defn.strategy is Strategies.any_flavour:
+            if defn.strategy.type_ is Strategies.any_flavour:
 
-                def supports_game_version(f: JsonDict):
+                def supports_game_version(f: CurseAddon_File):
                     return True
 
             else:
                 classic_version_prefix = '1.13'
                 flavour = 'wow_classic' if self.manager.config.is_classic else 'wow_retail'
 
-                def supports_game_version(f: JsonDict):
+                def supports_game_version(f: CurseAddon_File):
                     # Files can belong both to retail and classic
                     # but ``gameVersionFlavor`` can only be one of
                     # 'wow_retail' or 'wow_classic'.  To spice things up,
@@ -255,28 +332,27 @@ class CurseResolver(Resolver):
                         for v in f['gameVersion']
                     )
 
-            # 1 = stable; 2 = beta; 3 = alpha
-            if defn.strategy is Strategies.latest:
+            if defn.strategy.type_ is Strategies.latest:
 
-                def has_release_type(f: JsonDict):
+                def has_release_type(f: CurseAddon_File):
                     return True
 
-            elif defn.strategy is Strategies.curse_latest_beta:
+            elif defn.strategy.type_ is Strategies.curse_latest_beta:
 
-                def has_release_type(f: JsonDict):
+                def has_release_type(f: CurseAddon_File):
                     return f['releaseType'] == 2
 
-            elif defn.strategy is Strategies.curse_latest_alpha:
+            elif defn.strategy.type_ is Strategies.curse_latest_alpha:
 
-                def has_release_type(f: JsonDict):
+                def has_release_type(f: CurseAddon_File):
                     return f['releaseType'] == 3
 
             else:
 
-                def has_release_type(f: JsonDict):
+                def has_release_type(f: CurseAddon_File):
                     return f['releaseType'] == 1
 
-            def is_compatible(f: JsonDict):
+            def is_compatible(f: CurseAddon_File):
                 return is_not_libless(f) and supports_game_version(f) and has_release_type(f)
 
         if not files:
@@ -290,17 +366,9 @@ class CurseResolver(Resolver):
         except ValueError:
             raise E.PkgFileUnavailable(
                 f'no files compatible with {self.manager.config.game_flavour} '
-                f'using {defn.strategy.name!r} strategy'
+                f'using {defn.strategy.type_.name!r} strategy'
             )
 
-        # 1 = embedded library
-        # 2 = optional dependency
-        # 3 = required dependency
-        # 4 = tool
-        # 5 = incompatible
-        # 6 = include (wat)
-        deps = [m.PkgDep(id=d['addonId']) for d in file['dependencies'] if d['type'] == 3]
-
         return m.Pkg(
             source=self.source,
             id=metadata['id'],
@@ -311,15 +379,15 @@ class CurseResolver(Resolver):
             download_url=file['downloadUrl'],
             date_published=file['fileDate'],
             version=file['displayName'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
-            deps=deps,
+            options=m.PkgOptions(strategy=defn.strategy.type_.name),
+            deps=[m.PkgDep(id=d['addonId']) for d in file['dependencies'] if d['type'] == 3],
         )
 
     async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
         classic_version_prefix = '1.13'
         flavours = ('retail', 'classic')
 
-        def excise_compatibility(files: Any):
+        def excise_compatibility(files: List[CurseAddon_File]):
             for c in flavours:
                 if any(f['gameVersionFlavor'] == f'wow_{c}' for f in files):
                     yield c
@@ -337,7 +405,7 @@ class CurseResolver(Resolver):
                 gameId='1', sort=sort_order, pageSize=step, index=index
             )
             async with self.manager.web_client.get(url) as response:
-                json_response = await response.json()
+                json_response: List[CurseAddon] = await response.json()
 
             if not json_response:
                 break
@@ -357,6 +425,50 @@ class CurseResolver(Resolver):
                 )
 
 
+if TYPE_CHECKING:
+
+    class WowiCommonTerms(TypedDict):
+        UID: str  # Unique add-on ID
+        UICATID: str  # ID of category add-on is placed in
+        UIVersion: str  # Add-on version
+        UIDate: int  # Upload date expressed as unix epoch
+        UIName: str  # User-facing add-on name
+        UIAuthorName: str
+
+    class WowiCompatibilityEntry:
+        version: str  # Game version, e.g. '8.3.0'
+        name: str  # Xpac or patch name, e.g. "Visions of N'Zoth" for 8.3.0
+
+    class WowiListApiItem(WowiCommonTerms):
+        UIFileInfoURL: str  # Add-on page on WoWI
+        UIDownloadTotal: str  # Total number of downloads
+        UIDownloadMonthly: str  # Number of downloads in the last month and not 'monthly'
+        UIFavoriteTotal: str
+        UICompatibility: O[List[WowiCompatibilityEntry]]  # ``null`` if would be empty
+        UIDir: List[str]  # Names of folders contained in archive
+        UIIMG_Thumbs: O[List[str]]  # Thumbnail URLs; ``null`` if would be empty
+        UIIMGs: O[List[str]]  # Full-size image URLs; ``null`` if would be empty
+        # There are only two add-ons on the entire list with siblings
+        # (they refer to each other). I don't know if this was meant to capture
+        # dependencies (probably not) but it's so underused as to be worthless.
+        # ``null`` if would be empty
+        UISiblings: O[List[str]]
+        UIDonationLink: O[str]  # Absent from the first item on the list (!)
+
+    class WowiDetailsApiItem(WowiCommonTerms):
+        UIMD5: O[str]  # Archive hash, ``null` when UI is pending
+        UIFileName: str  # The actual filename, e.g. 'foo.zip'
+        UIDownload: str  # Download URL
+        UIPending: Literal['0', '1']  # Set to '1' if the file is awaiting approval
+        UIDescription: str  # Long description with BB Code and all
+        UIChangeLog: str  # This can also contain BB Code
+        UIHitCount: str  # Same as UIDownloadTotal
+        UIHitCountMonthly: str  # Same as UIDownloadMonthly
+
+    class WowiCombinedItem(WowiListApiItem, WowiDetailsApiItem):
+        pass
+
+
 class WowiResolver(Resolver):
     source = 'wowi'
     name = 'WoWInterface'
@@ -368,7 +480,7 @@ class WowiResolver(Resolver):
     # camelCase and some fields which were strings were converted to numbers.
     # Neither API provides access to classic files for multi-file add-ons and
     # 'UICompatibility' can't be relied on to enforce compatibility
-    # in instawow.  The API appears to inherit the version of the _latest_
+    # in instawow.  The API appears to inherit the version of the latest
     # file to have been uploaded, which for multi-file add-ons can be the
     # classic version.  Hoooowever the download link always points to the
     # 'retail' version, which for single-file add-ons belonging to the
@@ -376,10 +488,10 @@ class WowiResolver(Resolver):
     list_api_url = 'https://api.mmoui.com/v3/game/WOW/filelist.json'
     details_api_url = URL('https://api.mmoui.com/v3/game/WOW/filedetails/')
 
-    _files: O[JsonDict] = None
+    list_api_items: O[Dict[str, WowiListApiItem]] = None
 
     @staticmethod
-    def get_name_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> O[str]:
         url = URL(value)
         if (
             url.host in {'wowinterface.com', 'www.wowinterface.com'}
@@ -405,37 +517,38 @@ class WowiResolver(Resolver):
         from .manager import cache_json_response
 
         async with self.manager.locks['load WoWI catalogue']:
-            if self._files is None:
-                files = await cache_json_response(
+            if self.list_api_items is None:
+                list_api_items = await cache_json_response(
                     self.manager,
                     self.list_api_url,
                     3600,
                     label=f'Synchronising {self.name} catalogue',
                 )
-                self._files = {i['UID']: i for i in files}
+                self.list_api_items = {i['UID']: i for i in list_api_items}
 
-        ids_for_defns = {d: ''.join(takewhile(str.isdigit, d.name)) for d in defns}
-        numeric_ids = {i for i in ids_for_defns.values() if i.isdigit()}
+        defns_to_ids = {d: ''.join(takewhile(str.isdigit, d.alias)) for d in defns}
+        numeric_ids = {i for i in defns_to_ids.values() if i.isdigit()}
         url = self.details_api_url / f'{",".join(numeric_ids)}.json'
         try:
-            json_response = await cache_json_response(self.manager, url, 300)
+            details_api_items = await cache_json_response(self.manager, url, 300)
         except ClientResponseError as error:
             if error.status != 404:
                 raise
-            json_response = []
+            details_api_items = []
 
-        api_results = {r['UID']: {**self._files[r['UID']], **r} for r in json_response}
+        combined_items = {
+            r['UID']: cast('WowiCombinedItem', {**self.list_api_items[r['UID']], **r})
+            for r in details_api_items
+        }
         results = await gather(
-            self.resolve_one(d, api_results.get(i)) for d, i in ids_for_defns.items()
+            self.resolve_one(d, combined_items.get(i)) for d, i in defns_to_ids.items()
         )
         return dict(zip(defns, results))
 
-    async def resolve_one(self, defn: Defn, metadata: O[JsonDict]) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: O[WowiCombinedItem]) -> m.Pkg:
         if not metadata:
             raise E.PkgNonexistent
 
-        # 'UIPending' is set to '1' for files awaiting approval during which
-        # time 'UIMD5' is null - all other fields appear to be filled correctly
         return m.Pkg(
             source=self.source,
             id=metadata['UID'],
@@ -446,72 +559,116 @@ class WowiResolver(Resolver):
             download_url=metadata['UIDownload'],
             date_published=metadata['UIDate'],
             version=metadata['UIVersion'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            options=m.PkgOptions(strategy=defn.strategy.type_.name),
         )
 
     async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
         async with self.manager.web_client.get(self.list_api_url) as response:
-            json_response = await response.json()
+            list_api_items: List[WowiListApiItem] = await response.json()
 
-        for item in json_response:
+        for list_item in list_api_items:
             yield _CatalogueEntry(
                 source=self.source,
-                id=item['UID'],
-                name=item['UIName'],
-                folders=[item['UIDir']],
+                id=list_item['UID'],
+                name=list_item['UIName'],
+                folders=[list_item['UIDir']],
             )
 
 
+if TYPE_CHECKING:
+
+    class TukuiUi(TypedDict):
+        author: str
+        category: str
+        changelog: str
+        donate_url: str
+        downloads: int
+        git: str
+        id: Literal[-1, -2]  # -1 is Tukui and -2 ElvUI
+        lastdownload: str
+        lastupdate: str  # ISO date, e.g. '2020-02-02'
+        name: str
+        patch: str
+        screenshot_url: str
+        small_desc: str
+        ticket: str
+        url: str
+        version: str
+        web_url: str
+
+    class TukuiAddon(TypedDict):
+        author: str
+        category: str
+        changelog: str
+        donate_url: str
+        downloads: str
+        id: str
+        last_download: str
+        # ISO *datetime* with space sep and without an offset, e.g. '2020-02-02 12:12:20'
+        lastupdate: str
+        name: str
+        patch: str
+        screenshot_url: str
+        small_desc: str
+        url: str
+        version: str
+        web_url: str
+
+
 class TukuiResolver(Resolver):
     source = 'tukui'
     name = 'Tukui'
     strategies = {Strategies.default}
 
+    # There's also a ``/client-api.php`` endpoint which is apparently
+    # used by the Tukui client itself to check for updates for the two retail
+    # UIs only.  The response body appears to be identical to ``/api.php``
     api_url = URL('https://www.tukui.org/api.php')
 
     retail_uis = {'-1': 'tukui', '-2': 'elvui', 'tukui': 'tukui', 'elvui': 'elvui'}
 
     @staticmethod
-    def get_name_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> O[str]:
         url = URL(value)
         if url.host == 'www.tukui.org' and url.path in {
             '/addons.php',
             '/classic-addons.php',
             '/download.php',
         }:
-            name = url.query.get('id') or url.query.get('ui')
-            if name:
-                return name
+            alias = url.query.get('id') or url.query.get('ui')
+            if alias:
+                return alias
 
     async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
-        name = ui_name = self.retail_uis.get(defn.name)
-        if not name:
-            name = ''.join(takewhile('-'.__ne__, defn.name))
+        alias = ui_id = self.retail_uis.get(defn.alias)
+        if not alias:
+            alias = ''.join(takewhile('-'.__ne__, defn.alias))
 
         if self.manager.config.is_classic:
             query = 'classic-addon'
-        elif ui_name:
+        elif ui_id:
             query = 'ui'
         else:
             query = 'addon'
 
-        url = self.api_url.with_query({query: name})
+        url = self.api_url.with_query({query: alias})
         async with self.manager.web_client.get(url) as response:
+            # Does not 404 for non-existent add-ons but the response body is empty
             if not response.content_length:
                 raise E.PkgNonexistent
-            addon = await response.json(content_type=None)  # text/html
+            addon: Union[TukuiUi, TukuiAddon] = await response.json(content_type=None)  # text/html
 
         return m.Pkg(
             source=self.source,
             id=addon['id'],
-            slug=ui_name or slugify(f'{addon["id"]} {addon["name"]}'),
+            slug=ui_id or slugify(f'{addon["id"]} {addon["name"]}'),
             name=addon['name'],
             description=addon['small_desc'],
             url=addon['web_url'],
             download_url=addon['url'],
             date_published=datetime.fromisoformat(addon['lastupdate']),
             version=addon['version'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            options=m.PkgOptions(strategy=defn.strategy.type_.name),
         )
 
     async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
@@ -542,6 +699,29 @@ class TukuiResolver(Resolver):
                     )
 
 
+if TYPE_CHECKING:
+
+    # Not exhaustive (as you might've guessed).  Reference:
+    # https://docs.github.com/en/rest/reference/repos
+
+    class GithubRepo(TypedDict):
+        name: str  # the repo in user-or-org/repo
+        full_name: str  # user-or-org/repo
+        description: str
+        html_url: str
+
+    class GithubReleaseAsset(TypedDict):
+        name: str  # filename
+        content_type: str  # mime type
+        state: Literal['starter', 'uploaded']
+        browser_download_url: str
+
+    class GithubRelease(TypedDict):
+        tag_name: str  # Hopefully the version
+        published_at: str  # ISO datetime
+        assets: List[GithubReleaseAsset]
+
+
 class GithubResolver(Resolver):
     source = 'github'
     name = 'GitHub'
@@ -553,7 +733,7 @@ class GithubResolver(Resolver):
     repos_api_url = URL('https://api.github.com/repos')
 
     @staticmethod
-    def get_name_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> O[str]:
         url = URL(value)
         if url.host == 'github.com' and len(url.parts) > 2:
             return '/'.join(url.parts[1:3])
@@ -561,40 +741,41 @@ class GithubResolver(Resolver):
     async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
         """Resolve a hypothetical add-on hosted on GitHub.
 
-        The GitHub resolver, ahem, 'builds' on the work done by Torkus
-        (see https://github.com/ogri-la/strongbox/blob/develop/github-addons.md) -
-        it purports to support the different kinds of add-ons supported by strongbox
-        with the exception that instawow does not look for TOC files or validate
-        the contents of the ZIP file.
-        instawow will attempt to prioritise add-ons compatible with your selected
-        game flavour.  It will otherwise install the first file it encounters.
-        instawow will only install assets attached to releases.  It will not
-        install add-ons from VCS tarballs or 'zipballs' (i.e. from source).
+        The GitHub resolver is inspired by strongbox's
+        (see https://github.com/ogri-la/strongbox/blob/develop/github-addons.md)
+        and makes similar assumptions.
+        The repo must have releases.  The releases must have assets
+        attached to them.  instawow will not retrieve add-ons from VCS tarballs
+        or 'zipballs' (i.e. from source).
+        It will prioritise assets which appear to be compatible with the
+        selected game flavour.
+        It will *not* look for TOC files or validate the contents
+        of the ZIP file in any way.
         """
         from aiohttp import ClientResponseError
 
         from .manager import cache_json_response
 
-        repo_url = self.repos_api_url / defn.name
+        repo_url = self.repos_api_url / defn.alias
         try:
-            project_metadata = await cache_json_response(self.manager, repo_url, 3600)
+            project_metadata: GithubRepo = await cache_json_response(self.manager, repo_url, 3600)
         except ClientResponseError as error:
             if error.status == 404:
                 raise E.PkgNonexistent
             raise
 
-        if defn.strategy is Strategies.version:
-            release_url = repo_url / 'releases/tags' / defn.strategy_vals[0]
+        if defn.strategy.type_ is Strategies.version:
+            release_url = repo_url / 'releases/tags' / cast(VersionStrategy, defn.strategy).version
         else:
             release_url = repo_url / 'releases/latest'
         async with self.manager.web_client.get(release_url) as response:
             if response.status == 404:
                 raise E.PkgFileUnavailable('release not found')
-            release_metadata = await response.json()
+            release_metadata: GithubRelease = await response.json()
 
         try:
 
-            def is_valid_asset(asset: JsonDict):
+            def is_valid_asset(asset: GithubReleaseAsset):
                 return (
                     # There is something of a convention that Classic archives
                     # end in '-classic' and lib-less archives end in '-nolib'.
@@ -606,7 +787,7 @@ class GithubResolver(Resolver):
                     and asset['state'] == 'uploaded'
                 )
 
-            assets = release_metadata.get('assets', [])
+            assets = release_metadata['assets']
             matching_asset = next(
                 chain(
                     (
@@ -631,7 +812,7 @@ class GithubResolver(Resolver):
             download_url=matching_asset['browser_download_url'],
             date_published=release_metadata['published_at'],
             version=release_metadata['tag_name'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            options=m.PkgOptions(strategy=defn.strategy.type_.name),
         )
 
 
@@ -647,7 +828,7 @@ class InstawowResolver(Resolver):
 
     async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
         try:
-            source_id, slug = next(p for p in self._addons if defn.name in p)
+            source_id, slug = next(p for p in self._addons if defn.alias in p)
         except StopIteration:
             raise E.PkgNonexistent
 
@@ -671,5 +852,5 @@ class InstawowResolver(Resolver):
             download_url=builder.addon_file.as_uri(),
             date_published=datetime.now(),
             version=checksum[:7],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            options=m.PkgOptions(strategy=defn.strategy.type_.name),
         )
