@@ -4,20 +4,15 @@ from itertools import chain
 from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional as O, Sequence, Tuple, Type
 
 from loguru import logger
-from pydantic import BaseModel, Extra, ValidationError, validator
+from pydantic import BaseModel, Field, validator
 from yarl import URL
 
-from .config import BaseConfig
-from .utils import (
-    ManagerAttrAccessMixin,
-    bucketise,
-    dict_chain,
-    gather,
-    iter_in_thread as iit,
-    run_in_thread as t,
-)
+from .config import BaseConfig, Config as GlobalConfig
+from .utils import bucketise, chain_dict, gather, iter_in_thread as iit, run_in_thread as t
 
 if TYPE_CHECKING:
+    from pathlib import Path
+
     from .manager import Manager
 
     ImportString = str
@@ -29,13 +24,14 @@ import_api = URL('https://data.wago.io/api/raw/encoded')
 
 class BuilderConfig(BaseConfig):
     account: str
+    api_key: O[str]
 
-    class Config:
+    class Config:  # type: ignore
         env_prefix = 'WAC_'
 
-
-class WaConfigError(Exception):
-    pass
+    def get_saved_vars(self, global_config: GlobalConfig) -> Path:
+        root_for_flavour = global_config.addon_dir.parents[1]
+        return root_for_flavour / 'WTF' / 'Account' / self.account / 'SavedVariables'
 
 
 class WeakAura(BaseModel):
@@ -44,12 +40,9 @@ class WeakAura(BaseModel):
     parent: O[str]
     url: URL
     version: int
-    semver: O[str]
-    ignore_wago_update: bool = False
 
     class Config:
         arbitrary_types_allowed = True
-        fields = {'ignore_wago_update': 'ignoreWagoUpdate'}
 
     @validator('url', pre=True)
     def _convert_url(cls, value: str) -> URL:
@@ -77,15 +70,12 @@ class WeakAuras(BaseModel):
 
 
 class Plateroo(WeakAura):
+    id: str = Field(alias='Name')
     uid = ''
 
-    class Config:
-        arbitrary_types_allowed = True
-        fields = {'id': 'Name'}
-
 
 class Plateroos(WeakAuras):
-    class Meta:
+    class Meta:  # type: ignore
         model = Plateroo
         filename = 'Plater.lua'
         table_prefix = 'PlaterDB'
@@ -109,42 +99,30 @@ class _WagoChangelog(BaseModel):
     format: O[str]
     text: str = ''
 
-    class Config:
-        extra = Extra.forbid
-
 
 class WagoResponse(BaseModel):
-    id: str
+    id: str = Field(alias='_id')
     name: str  # +
     slug: str  # +
     url: str
     created: str
     modified: str
     game: str
-    fork_of: O[str]
+    fork_of: O[str] = Field(alias='forkOf')
     username: str  # +
     version: int  # +
-    version_string: str  # +
+    version_string: str = Field(alias='versionString')  # +
     changelog: _WagoChangelog  # +
-    region_type: O[str]
-
-    class Config:
-        extra = Extra.forbid
-        fields = {
-            'id': '_id',
-            'fork_of': 'forkOf',
-            'version_string': 'versionString',
-            'region_type': 'regionType',
-        }
+    region_type: O[str] = Field(alias='regionType')
 
 
-class WaCompanionBuilder(ManagerAttrAccessMixin):
+class WaCompanionBuilder:
     """A WeakAuras Companion port for shellfolk."""
 
-    def __init__(self, manager: Manager, account: O[str] = None) -> None:
+    def __init__(self, manager: Manager, builder_config: BuilderConfig) -> None:
         self.manager = manager
-        self.addon_file = self.config.plugin_dir / __name__ / 'WeakAurasCompanion.zip'
-        self.account = account
+        self.addon_file = self.manager.config.plugin_dir / __name__ / 'WeakAurasCompanion.zip'
+        self.builder_config = builder_config
 
     @staticmethod
     def extract_auras(model: Type[WeakAuras], source: str) -> WeakAuras:
@@ -168,18 +146,7 @@ class WaCompanionBuilder(ManagerAttrAccessMixin):
     def extract_installed_auras(self) -> Iterator[WeakAuras]:
         import time
 
-        try:
-            builder_config = BuilderConfig(account=self.account)
-        except ValidationError as error:
-            raise WaConfigError from error
-
-        saved_vars = (
-            self.config.addon_dir.parents[1]
-            / 'WTF'
-            / 'Account'
-            / builder_config.account
-            / 'SavedVariables'
-        )
+        saved_vars = self.builder_config.get_saved_vars(self.manager.config)
         for model in WeakAuras, Plateroos:
             file = saved_vars / model.Meta.filename
             if not file.exists():
@@ -189,22 +156,27 @@ class WaCompanionBuilder(ManagerAttrAccessMixin):
                 aura_groups = self.extract_auras(
                     model, file.read_text(encoding='utf-8-sig', errors='replace')
                 )
-                logger.debug(f'auras extracted in {time.perf_counter() - start}s')
+                logger.debug(f'{model.__name__} extracted in {time.perf_counter() - start}s')
                 yield aura_groups
 
     async def get_wago_metadata(self, aura_groups: WeakAuras) -> List[WagoResponse]:
         aura_ids = list(aura_groups.entries)
         url = aura_groups.Meta.api.with_query(ids=','.join(aura_ids))
-        async with self.web_client.get(url) as response:
+        async with self.manager.web_client.get(
+            url, headers={'api-key': self.builder_config.api_key or ''}
+        ) as response:
             metadata = await response.json()
 
-        results = dict_chain(
+        results = chain_dict(
             aura_ids, None, ((i.slug, i) for i in map(WagoResponse.parse_obj, metadata))
         )
         return list(results.values())
 
     async def get_wago_import_string(self, aura_id: str) -> str:
-        async with self.web_client.get(import_api.with_query(id=aura_id)) as response:
+        async with self.manager.web_client.get(
+            import_api.with_query(id=aura_id),
+            headers={'api-key': self.builder_config.api_key or ''},
+        ) as response:
             return await response.text()
 
     async def get_remote_auras(
@@ -234,7 +206,7 @@ class WaCompanionBuilder(ManagerAttrAccessMixin):
             lstrip_blocks=True,
             loader=FunctionLoader(partial(read_text, wa_templates)),
         )
-        aura_dict = dict_chain((WeakAuras, Plateroos), [], auras)
+        aura_dict = chain_dict((WeakAuras, Plateroos), [], auras)
 
         self.addon_file.parent.mkdir(exist_ok=True)
         with ZipFile(self.addon_file, 'w') as file:
@@ -301,20 +273,15 @@ class WaCompanionBuilder(ManagerAttrAccessMixin):
             write_tpl('init.lua', {})
             write_tpl(
                 'WeakAurasCompanion.toc',
-                {'interface': '11305' if self.config.is_classic else '80300'},
+                {'interface': '11305' if self.manager.config.is_classic else '80300'},
             )
 
     async def build(self) -> None:
-        iter_auras = iit(self.extract_installed_auras())
-        aura_groups = [
-            g.__class__(
-                entries={
-                    k: v for k, v in g.entries.items() if not any(i.ignore_wago_update for i in v)
-                }
-            )
-            async for g in iter_auras
-        ]
-        remote_auras = await gather((self.get_remote_auras(g) for g in aura_groups), False)
+        installed_auras = iit(self.extract_installed_auras())
+        remote_auras = await gather(
+            [self.get_remote_auras(g) async for g in installed_auras],
+            False,
+        )
         await t(self.make_addon)(remote_auras)
 
     def checksum(self) -> str:
