@@ -5,7 +5,7 @@ import re
 import pytest
 
 from instawow.config import Config
-from instawow.manager import Manager, init_web_client, prepare_db_session
+from instawow.manager import Manager
 from instawow.utils import get_version
 
 
@@ -62,18 +62,9 @@ def full_config(tmp_path, partial_config):
 
 
 @pytest.fixture
-async def web_client():
-    async with init_web_client() as web_client:
-        yield web_client
-
-
-@pytest.fixture
-def manager(full_config, web_client):
+def manager(full_config):
     config = Config(**full_config).write()
-    db_session = prepare_db_session(config=config)
-
-    manager = Manager(config, db_session)
-    manager.web_client = web_client
+    manager = Manager.from_config(config)
     yield manager
 
 
@@ -83,6 +74,7 @@ def JsonResponse(aresponses):
 
 
 @pytest.fixture
+@should_mock
 def mock_pypi(aresponses, JsonResponse):
     aresponses.add(
         'pypi.org',
@@ -285,8 +277,12 @@ def mock_github(aresponses, JsonResponse):
     )
 
 
-@pytest.fixture(autouse=True)
+@pytest.fixture
 def mock_all(
-    mock_pypi, mock_curse, mock_wowi, mock_tukui, mock_github,
+    mock_pypi,
+    mock_curse,
+    mock_wowi,
+    mock_tukui,
+    mock_github,
 ):
     pass
